#include "mainwindow.h"

int completed;

bool useproxy;
QString proxy;
int proxyport;
QString proxyuser;
QString proxypass;

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{

#ifndef CODEC_LOCALE
#error Your OS locale is undefined! Check mainwindow.h and add your OS and locale.
#endif

    QTextCodec *codecTr = QTextCodec::codecForName(CODEC_TR);
    QTextCodec *codecLocale = QTextCodec::codecForName(CODEC_LOCALE);
    QTextCodec *codecCStrings = QTextCodec::codecForName(CODEC_LOCALE);
    QTextCodec::setCodecForTr(codecTr);
    QTextCodec::setCodecForLocale(codecLocale);
    QTextCodec::setCodecForCStrings(codecCStrings);

    model = new QStandardItemModel;
    view = new QTreeView;
    view->setModel(model);
    view->setSortingEnabled(true);

    connect(view,SIGNAL(doubleClicked(const QModelIndex&)),this,SLOT(open(const QModelIndex&)));

    setupMenus();
    setupWidgets();

    settingswindow=new SettingsWindow(this);
//    settingswindow->setWindowModality(Qt::WindowModal);
    settingswindow->setWindowTitle(tr(""));
    settingswindow->setWindowIcon(QPixmap(":/images/logo.png"));
    settingswindow->resize(1,1);
}

MainWindow::~MainWindow()
{
    settingswindow->close();
    delete settingswindow;
}

void MainWindow::setupMenus()
{
    QMenu *fileMenu = menuBar()->addMenu(tr("&"));

    QAction *saveall2fileAction = fileMenu->addAction(tr(" & URL..."));
    saveall2fileAction->setShortcut(QKeySequence(tr("Ctrl+A")));
    QAction *saveselected2fileAction = fileMenu->addAction(tr(" & URL..."));
    saveselected2fileAction->setShortcut(QKeySequence(tr("Ctrl+S")));
    QAction *loadfromfileAction = fileMenu->addAction(tr("& URL..."));
    loadfromfileAction->setShortcut(QKeySequence(tr("Ctrl+L")));

    QMenu *clipboardMenu = menuBar()->addMenu(tr("&Clipboard"));

    QAction *saveall2clpbrdAction = clipboardMenu->addAction(tr(" & URL!"));
    saveall2clpbrdAction->setShortcut(QKeySequence(tr("Ctrl+C")));
    QAction *saveselected2clpbrdAction = clipboardMenu->addAction(tr(" & URL!"));
    saveselected2clpbrdAction->setShortcut(QKeySequence(tr("Ctrl+P")));
    QAction *loadfromclpbrdAction = clipboardMenu->addAction(tr("& URL!"));
    loadfromclpbrdAction->setShortcut(QKeySequence(tr("Ctrl+B")));

    QMenu *processMenu = menuBar()->addMenu(tr("&"));
    QAction *goAction = processMenu->addAction(tr("  !"));
    goAction->setShortcut(QKeySequence(tr("Ctrl+G")));
    QAction *selectAction = processMenu->addAction(tr("..."));
    selectAction->setShortcut(QKeySequence(tr("Ctrl+T")));

    QAction *settingsAction = menuBar()->addAction(tr("&"));
    settingsAction->setShortcut(QKeySequence(tr("Ctrl+T")));

    QAction *exitAction = menuBar()->addAction(tr("&!"));
    exitAction->setShortcut(QKeySequence(tr("Ctrl+Q")));

    QMenu *helpMenu = menuBar()->addMenu(tr("&"));
    QAction *aboutAction = helpMenu->addAction(tr("& ..."));
    aboutAction->setShortcut(QKeySequence(tr("Ctrl+H")));
    QAction *aboutQtAction = helpMenu->addAction(tr(" &Qt..."));
    aboutQtAction->setShortcut(QKeySequence(tr("Ctrl+T")));

    connect(saveall2fileAction, SIGNAL(triggered()), this, SLOT(saveall2file()));
    connect(saveselected2fileAction, SIGNAL(triggered()), this, SLOT(savesel2file()));
    connect(loadfromfileAction, SIGNAL(triggered()), this, SLOT(loadfromfile()));
    connect(saveall2clpbrdAction, SIGNAL(triggered()), this, SLOT(saveall2clpbrd()));
    connect(saveselected2clpbrdAction, SIGNAL(triggered()), this, SLOT(savesel2clpbrd()));
    connect(loadfromclpbrdAction, SIGNAL(triggered()), this, SLOT(loadfromclpbrd()));
    connect(goAction, SIGNAL(triggered()), this, SLOT(go()));
    connect(selectAction, SIGNAL(triggered()), this, SLOT(select()));
    connect(settingsAction, SIGNAL(triggered()), this, SLOT(settings()));
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));

}

void MainWindow::setupWidgets()
{

//    connect(...button, SIGNAL(clicked()), this, SLOT(...()));


//    connect(listbox, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(status(QListWidgetItem*)));
//    connect(listbox, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(status(QListWidgetItem*)));
 	
    stat=new QStatusBar;
    progress=new QProgressBar;

    QFrame *frame = new QFrame;
    QGridLayout *layout = new QGridLayout(frame);
    layout->setMargin (0);
    layout->addWidget(view, 1, 0);
    layout->addWidget(progress, 2, 0);
    layout->addWidget(stat, 3, 0);
    setCentralWidget(frame);

    setWindowTitle(tr(name));
    setWindowIcon(QPixmap(":/images/logo.png"));
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(this, tr(""),
                                    tr(" ?"),
                                    QMessageBox::Yes | QMessageBox::No);
    if (reply == QMessageBox::Yes)
	event->accept();
    else
	event->ignore();

}

void MainWindow::saveall2clpbrd()
{
	QModelIndex index;
        QString text;
	for(int i=0;i<model->rowCount();i++)
	{
		index=model->index(i,0);
		text+=index.data().toString()+"\n";
	}

	QClipboard *clipboard = QApplication::clipboard();
	clipboard->setText(text);	
}

void MainWindow::saveall2file()
{
	QString fileName = QFileDialog::getSaveFileName(this,tr("SaveToFile"),"",tr("Text file (*);;Text Files (*.txt)"));
	if (fileName.length()!=0)
	{
		QFile file(fileName);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
		{
			QMessageBox::critical(0,qApp->tr("!"),
			qApp->tr("\n  .")+
			qApp->tr("\n Cancel"), QMessageBox::Cancel);
		}
		else
		{
	    		QTextStream out(&file);
			QModelIndex index;
			for(int i=0;i<model->rowCount();i++)
			{
				index=model->index(i,0);
      	        		out<<index.data().toString()<<endl;
			}
		}
	}
}


void MainWindow::savesel2clpbrd()
{
	QModelIndex index;
        QString text;
	for(int i=0;i<model->rowCount();i++)
	{
		index=model->index(i,0);
		if(view->selectionModel()->isSelected(index))
			text+=index.data().toString()+"\n";
	}

	QClipboard *clipboard = QApplication::clipboard();
	clipboard->setText(text);	
}

void MainWindow::savesel2file()
{
	QString fileName = QFileDialog::getSaveFileName(this,tr("SaveToFile"),"",tr("Text file (*);;Text Files (*.txt)"));
	if (fileName.length()!=0)
	{
		QFile file(fileName);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
		{
			QMessageBox::critical(0,qApp->tr("!"),
			qApp->tr("\n  .")+
			qApp->tr("\n Cancel"), QMessageBox::Cancel);
		}
		else
		{
	    			QTextStream out(&file);
			QModelIndex index;
			for(int i=0;i<model->rowCount();i++)
			{
				index=model->index(i,0);
				if(view->selectionModel()->isSelected(index))
	      	        		out<<index.data().toString()<<endl;
			}
		}
	}
}

void MainWindow::load(const QStringList& items)
{
	model->clear();
        QStringList::const_iterator constIterator;
	QStandardItem *item;
        for (constIterator = items.constBegin(); constIterator != items.constEnd(); ++constIterator)
 	{
		QList<QStandardItem *> row;
		QString url=(*constIterator).toLocal8Bit().constData();
		if(url.length()==0)
			continue;
         	item=new QStandardItem(url.trimmed());
		row<<item;
         	item=new QStandardItem("N/A");
		row<<item;
         	item=new QStandardItem("N/A");
		row<<item;
         	item=new QStandardItem("N/A");
		row<<item;
         	item=new QStandardItem("N/A");
		row<<item;
		model->appendRow(row);
	}
	model->setHeaderData(0, Qt::Horizontal, tr("url"))  ;
	model->setHeaderData(1, Qt::Horizontal, tr("dsc"))  ;
	model->setHeaderData(2, Qt::Horizontal, tr("sqrt(max(dsc,7))"))  ;
	model->setHeaderData(3, Qt::Horizontal, tr("encoding"))  ;
	model->setHeaderData(4, Qt::Horizontal, tr("whois"))  ;
	view->resizeColumnToContents(0);
	if(view->columnWidth(0)>600)
		view->setColumnWidth(0,600);
	view->setColumnWidth(1,70);
	view->setColumnWidth(2,120);
	view->setColumnWidth(3,120);
	view->setColumnWidth(4,60);
//	view->resizeColumnToContents(...);
	view->setSelectionBehavior(QAbstractItemView::SelectRows);
	view->setSelectionMode(QAbstractItemView::MultiSelection);
}
void MainWindow::loadfromclpbrd()
{

	QClipboard *clipboard = QApplication::clipboard();
	QString text=clipboard->text();	
        QStringList list = text.split("\n", QString::SkipEmptyParts);

	load(list);

}
void MainWindow::loadfromfile()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),"",tr("Text (*.txt)"));
	if (fileName.length()!=0)
	{
		QFile file(fileName);
		QStringList list;
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		{
			QMessageBox::critical(0,qApp->tr("!"),
			qApp->tr("\n    .")+
			qApp->tr("\n Cancel"), QMessageBox::Cancel);
		}
		else
		{
			QTextStream in(&file);
			while (!in.atEnd())
			{
				list<<in.readLine();
			}
			load(list);
		}
	}
}

void MainWindow::updateurl(const int i)
{
	Thread* httpthread=new Thread(model,view,mutex,i,this);

	connect(httpthread,SIGNAL(status(QString,int)),this,SLOT(status(QString,int)));

	while((httpthread->thrcount)>=HTTP_MAX_THREADS)
		qApp->processEvents();

	httpthread->start();

//	QFile debugfile;
//	debugfile.setFileName("thr.out"+sssss);
//	debugfile.open(QIODevice::WriteOnly | QIODevice::Text);
//        QTextStream debug(&debugfile);
//	debug<<"started"<<httpthread->thrcount;
}


void MainWindow::go()
{
	if( settingswindow->proxy->text().toStdString().length() && settingswindow->port->text().toStdString().length() )
	{
		useproxy=true;
		proxy=settingswindow->proxy->text();
		proxyport=settingswindow->port->text().toInt();
		proxyuser=settingswindow->proxyuser->text();
		proxypass=settingswindow->proxypass->text();
	}
	else
		useproxy=false;

	total=model->rowCount();
	completed=0;
	progress->setMinimum(0);
	progress->setMaximum(total);
	for(int i=0;i<model->rowCount();i++)
		updateurl(i);
}

void MainWindow::select()
{
	bool ok;

//	QString s=QInputDialog::getText(this, tr("Test:"),tr(""), QLineEdit::Normal, "", &ok);
//
//	tstr q=s.toStdString();
//	tnorm norm;
//	q=norm[q];
//	s=q.c_str();
//
//	QMessageBox::critical(0,qApp->tr("!"),
//	s+qApp->tr("\n Cancel"), QMessageBox::Cancel);

	int threshold=QInputDialog::getInteger(this, tr(" dsc:"),tr("  DSC   "), 7, 1, 2147483647, 1, &ok);
	if(ok)
		for(int i=0;i<model->rowCount();i++)
		{
			QModelIndex indexl=model->index(i,0);
			QModelIndex indexr=model->index(i,4);
			QModelIndex index=model->index(i,1);
			QItemSelection selection(indexl,indexr);
			if(index.data().toInt()>threshold)
		        	view->selectionModel()->select(selection,QItemSelectionModel::Select);
			else
		        	view->selectionModel()->select(selection,QItemSelectionModel::Deselect);
		}
}

void MainWindow::about()
{
    QString content=copyright;
    content+="\n";
    content+=OS;
    content+="/QT ";
    content+=QT_VERSION_STR;
    QMessageBox::information(this, tr(name) , tr(content.toLocal8Bit().data()));
}

void MainWindow::settings()
{
	settingswindow->show();
}

void MainWindow::status(QString s,int i)
{
	QMutexLocker locker(&statusmutex);
	QString id; id.setNum(i+1);
//	QString id; id.setNum(completed);
	QString tot; tot.setNum(total);
	progress->setValue(completed);

	s=tr(" ")+id+tr("  ")+tot+tr(", : ")+s;

	stat->showMessage(s);
}

void MainWindow::open(const QModelIndex& what)
{
	QString url=what.data().toString();

#ifdef _WIN32
	QProcess openPage;
        //openPage.startDetached( "cmd", QStringList() << "/c" << "start" << url.toString() );
        //openPage.waitForFinished();

        openPage.execute( QString( "rundll32 url.dll,FileProtocolHandler \"%1\"" ).arg( url ) );
#endif
}

Thread::Thread(QStandardItemModel *givenmodel, QTreeView *givenview, QMutex& givenmutex, int id, QObject *parent)
    : QThread(parent), model(givenmodel), view(givenview), mutex(givenmutex), i(id)
{
	thrcount++;
}

int Thread::thrcount=0;

void Thread::run()
{
	mutex.lock();

	QModelIndex index=model->index(i,0);

	QUrl url(index.data().toString());

	connect(&http, SIGNAL(requestFinished(int, bool)), this, SLOT(httpRequestFinished(int, bool)));

	QHttp::ConnectionMode mode = url.scheme().toLower() == "https" ? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp;
	http.setHost(url.host(), mode, url.port() == -1 ? 0 : url.port());

	if(useproxy)
		if(proxyuser.length())
			http.setProxy(proxy,proxyport,proxyuser,proxypass);
		else
			http.setProxy(proxy,proxyport);

	QHttpRequestHeader header("GET",url.toString());
	header.setValue("Host", url.host());
	header.setValue( "User-Agent", "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/0.4.154.29 Safari/525.19 NauseaChecker/1.1b");
	reqid=http.request(header);

//	reqid=http.get(url.toString());
	connect(&timer, SIGNAL(timeout()), this, SLOT(httpRequestTimeout()));
	timer.setSingleShot(true);
	timer.start(HTTP_TIMEOUT*1000);
	model->setData(index,url.toString());
	mutex.unlock();

	this->exec();

}


void Thread::httpRequestFinished(int requestId, bool error)
{

	if(reqid!=requestId)
	{
		return;
	}

	completed++;

//	QMutexLocker locker(&mutex);
	mutex.lock();
	QModelIndex index0=model->index(i,0);
	QModelIndex index1=model->index(i,1);
	QModelIndex index2=model->index(i,2);
	QModelIndex index3=model->index(i,3);
	QModelIndex index4=model->index(i,4);
	int idforstatus=i;
	mutex.unlock();

	if(error)
	{
		mutex.lock();
		{ QWaitCursor cursor; model->setData(index1,"ERROR"); }
		emit status("url: "+index0.data().toString(),idforstatus);
		thrcount--;
		mutex.unlock();
		quit();
		return ;
	}


	QString encoding=CODEC_LOCALE;
	if(http.lastResponse().hasKey("content-type"))
	{
		encoding=http.lastResponse().value("content-type");
		encoding.replace(0,encoding.indexOf("=")+1,"");
	}
	

	QString s=http.readAll().data();

	s=s.simplified();

	if(s.length())
	{
		QTextCodec *codec;

		QString encodingname;

		if(encoding=="text/html")
		    encodingname="CP1251";
		else
		    encodingname=encoding;
		    
		if(codec = QTextCodec::codecForName(encodingname.toStdString().c_str()))
		{
			QString string=codec->toUnicode(s.toStdString().c_str());
			codec = QTextCodec::codecForName(CODEC_LOCALE);
			s = codec->fromUnicode(string).data();
		}

		tdocument doc(s.toStdString());
		pvector p=doc.getsorted();
	
	   	mutex.lock();
		{ QWaitCursor cursor; model->itemFromIndex(index0)->setRowCount(0); }
	   	mutex.unlock();

		int c=0;

	        if(p.size()==0)
		{
		   	mutex.lock();
			{ QWaitCursor cursor; model->setData(index1,"NO WORDS"); }
			emit status("url: "+index0.data().toString(),idforstatus);
			thrcount--;
		   	mutex.unlock();
			quit();
			return ;
	   	}

		for(pvector::iterator i=p.begin();i!=p.end()&&c<MAX_WORDS_ATTACHED;i++,c++)
		{
			QStandardItem *item;
			QList<QStandardItem *> row;
	         	item=new QStandardItem(i->word.c_str());
			row<<item;
			QString number;
			number.setNum(i->count);
	         	item=new QStandardItem(number); //
			row<<item;
			number.setNum(sqrt(i->count>7?i->count:7));
	         	item=new QStandardItem(number);
			row<<item;
		   	mutex.lock();
			{ QWaitCursor cursor;  model->itemFromIndex(index0)->appendRow(row); }
		   	mutex.unlock();

			int cc=0;
			pvector q=doc.getforms(i->word);
			for(pvector::iterator j=q.begin();j!=q.end()&&cc<MAX_FORMS_ATTACHED;j++,cc++)
			{
				QStandardItem *item;
				QList<QStandardItem *> row;
		         	item=new QStandardItem(j->word.c_str());
				row<<item;
				QString number;
				number.setNum(j->count);
		         	item=new QStandardItem(number); //
				row<<item;
				number.setNum(sqrt(j->count>7?j->count:7));
		         	item=new QStandardItem(number);
				row<<item;
			   	mutex.lock();
				{ QWaitCursor cursor; model->itemFromIndex(index0)->child(c,0)->appendRow(row); }
			   	mutex.unlock();
			}
		}
	
	   	mutex.lock();
		{
			QWaitCursor cursor; 
			model->setData(index1,p[0].count);
			model->setData(index2,sqrt(p[0].count>7?p[0].count:7));
			model->setData(index3,encoding);
			view->collapse(index0);
			view->update();
		}
	   	mutex.unlock();

		emit status("url: "+index0.data().toString(),idforstatus);
	}
	else
	{
	   	mutex.lock();
		{ QWaitCursor cursor; model->setData(index1,"EMPTY"); }
		emit status("url: "+index0.data().toString(),idforstatus);
	   	mutex.unlock();
	}

   	mutex.lock();
	thrcount--;
   	mutex.unlock();
	quit();

}

void Thread::httpRequestTimeout()
{
	if(http.state() == QHttp::Connecting) 
		http.abort();
}
